@@ -203,8 +203,8 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
                     off_t nr = file.readBlock (buf, sizeof (buf) - 1);
                     if (nr > 0) {
                         buf[nr] = 0;
-                        String mimestr = MimeType::findByContent (buf, nr);
-                        if (!mimestr.startsWith (String ("text/")) ||
+                        int sz = nr > 512 ? 512 : nr;
+                        if (MimeType::isBufferBinaryData (buf, sz) ||
                                 !strncmp (buf, "RIFF", 4))
                             maybe_playlist = false;
                     }
@@ -352,6 +352,11 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
 void MediaInfo::setMimetype (const String &mt)
 {
     mime = mt;
+
+    Mrl *mrl = node ? node->mrl () : NULL;
+    if (mrl && mrl->mimetype.isEmpty ())
+        mrl->mimetype = mt;
+
     if (!mt.isEmpty () && ActorAgent::Any == type) {
         if (mimetype ().startsWith ("image/"))
             type = ActorAgent::Image;
@@ -399,7 +404,9 @@ KDE_NO_EXPORT void MediaInfo::create () {
                     }
                 }
                 if (data.size () &&
-                        (!(mimetype ().startsWith ("text/") ||
+                        (!(!MimeType::isBufferBinaryData (
+                                data.data (),
+                                data.size () > 512 ? 512 : data.size ()) ||
                            mime == "image/vnd.rn-realpix") ||
                          !readChildDoc ()))
                     media = new ImageActor (mgr, node, url, data);
@@ -432,8 +439,8 @@ static bool validDataFormat (ActorAgent::ActorType type, const ByteArray &ba) {
                 (ba.size () > 4 &&
                  !strncmp (ba.data (), "RIFF", 4))); // falsely seen as text
         if (!kill && ba.size() > 14) {
-            String ct = MimeType::findByContent (ba.data(), sz);
-            kill = !ct.startsWith ("text/") ||
+            int sz = ba.size () > 512 ? 512 : ba.size ();
+            kill = MimeType::isBufferBinaryData (ba.data(), sz) ||
                 (!strncasecmp (ba.data (), "<html", 5) || // html admin
                  !strncasecmp (ba.data (), "<!doctype html", 14));
         }
@@ -445,14 +452,15 @@ static bool validDataFormat (ActorAgent::ActorType type, const ByteArray &ba) {
 }
 
 KDE_NO_EXPORT void MediaInfo::jobResult (IOJob *jb) {
-    if (jb->error () || (data.size () < 512 && !validDataFormat (type, data)))
+    if (jb->error ())
         data.resize (0);
     job = 0L; // signal KIO::IOJob::result deletes itself
     if (mime.isEmpty ()) {
-        if (ActorAgent::Any == type && data.size () >= 512)
-            setMimetype (MimeType::findByContent (data.data(), 512));
-        else
-            setMimetype (jb->contentType ());
+        if (data.size () > 0) {
+            setMimetype (MimeType::findByContent (data.data(), data.size ()));
+            if (!validDataFormat (type, data))
+                data.resize (0);
+        }
         if (node->mrl ()) {
             node->mrl ()->mimetype = mime;
             node->mrl ()->audio_only = isAudioOnlyMime (mime);
@@ -471,8 +479,7 @@ KDE_NO_EXPORT void MediaInfo::jobData (IOJob *jb, ByteArray &ba) {
         memcpy (data.data () + old_size, ba.data (), ba.size ());
 
         if (old_size < 512 && newsize >= 512) {
-            if (ActorAgent::Any == type)
-                setMimetype (MimeType::findByContent (data.data (), 512));
+            setMimetype (MimeType::findByContent (data.data (), 512));
             if (!validDataFormat (type, data)) {
                 data.resize (0);
                 job->kill (false); // not quiet, wants jobResult
@@ -527,7 +534,7 @@ void AudioVideoActor::pause () {
 void AudioVideoActor::unpause () {
     Process *process = m_agent->control ()->process ();
     if (process && process->playing ())
-        process->pause ();
+        process->play (NULL);
 }
 
 //------------------------%<----------------------------------------------------
