@@ -845,9 +845,9 @@ static gboolean cb_finished (gpointer data) {
         if (gst_bus_async)
             g_signal_handler_disconnect (gst_bus, gst_bus_async);
         gst_object_unref (gst_bus);
-        if (StatePlaying != transition_state)
-            /* work around crash in libpulse.so.0(pa_context_disconnect */
-            gst_object_unref (GST_OBJECT (gst_elm_play));
+        /*if (StatePlaying != transition_state)
+            / * work around crash in libpulse.so.0(pa_context_disconnect * /
+            gst_object_unref (GST_OBJECT (gst_elm_play));*/
         gst_bus = 0L;
         gst_elm_play = 0L;
         color_balance = 0L;
@@ -964,6 +964,7 @@ int main (int argc, char **argv) {
     display = XOpenDisplay(NULL);
     screen  = XDefaultScreen(display);
 
+    g_set_application_name ("FMRadio"); /* BR:6694*/
     gst_init (NULL, NULL);
 
     event_loop = g_main_loop_new (0L, FALSE);
