@@ -5,6 +5,7 @@
 #include <locale.h>
 #include <errno.h>
 #include <libintl.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -25,10 +26,8 @@
 #include <hildon/hildon-file-chooser-dialog.h>
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-pannable-area.h>
+#include <hildon/hildon-gtk.h>
 
-#define bool char
-#define true 1
-#define false 0
 #include "kmplayer.h"
 /*#define noname www_xpm
 //#include "www.xpm"
@@ -371,7 +370,9 @@ static void create_toolbar (Application * app) {
     app->tb_pause = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_PAUSE);
     app->tb_next = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_NEXT);
     app->tb_stop = gtk_tool_button_new_from_stock (GTK_STOCK_MEDIA_STOP);
-    app->tb_save = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE);
+    /*app->tb_save = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE);*/
+    app->tb_save = gtk_tool_button_new (gtk_image_new_from_icon_name(
+                "notes_save", HILDON_ICON_SIZE_TOOLBAR), NULL);
     app->tb_go = gtk_tool_button_new (gtk_image_new_from_icon_name(
                 "widgets_tickmark_grid", HILDON_ICON_SIZE_TOOLBAR), NULL);
     app->tb_item_playlist_commit = GTK_WIDGET (gtk_tool_button_new (
@@ -537,7 +538,7 @@ gint dbus_req_handler( const gchar *interface, const gchar *method,
                 OpenLocation *ol = (OpenLocation *)g_malloc0 (sizeof (OpenLocation));
                 ol->filename = g_strdup(val.value.s);
                 ol->app = appdata;
-                g_idle_add (delayed_open_location, (gpointer)ol);
+                g_idle_add ((GSourceFunc)delayed_open_location, (gpointer)ol);
             }
         }
     } else
@@ -711,6 +712,7 @@ int main (int argc, char* argv[]) {
     pango_cxt = gtk_widget_get_pango_context (app->infopanel_view);
     pfd = pango_font_description_from_string ("Sans 12");
     gtk_widget_modify_font (app->infopanel_view, pfd);
+    pango_font_description_free (pfd);
     app->html_infopanel_view = gtk_html_new ();
     gtk_container_add (GTK_CONTAINER (app->html_viewport),
             app->html_infopanel_view);
@@ -779,7 +781,7 @@ int main (int argc, char* argv[]) {
                 ol->filename = g_strdup (argv[i]);
         }
         ol->app = app;
-        g_idle_add (delayed_open_location, (gpointer)ol);
+        g_idle_add ((GSourceFunc)delayed_open_location, (gpointer)ol);
     }
     gtk_main ();
     save_locationbar (app);
