@@ -899,6 +899,7 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             playlist_data->document->defer ();
     } else if (ns == Process::Playing) {
         mrl->audio_only |= !p->hasVideo ();
+        mrl->message (MsgMediaStarted, NULL);
         if (mrl->state == Element::state_deferred)
             mrl->undefer ();
         setLoading (100); // move possible buffer info dialog
@@ -1411,8 +1412,7 @@ void app_set_suspend (Application *a, bool suspend) {
     if (pd->document &&
             pd->document->active () &&
             (!CONTROL(a)->process ()->playing () ||
-             CONTROL(a)->process()->mrl()->view_mode ==Mrl::WindowMode ||
-             CONTROL(a)->process ()->hasVideo ())) {
+             CONTROL(a)->process()->mrl()->view_mode == Mrl::WindowMode)) {
         if (pd->document->state == Node::state_deferred) {
             if (!suspend)
                 pd->document->undefer ();
