@@ -694,7 +694,8 @@ Mrl::Mrl (NodePtr & d, short id)
       clip_begin (0),
       view_mode (SingleMode),
       resolved (false), bookmarkable (true),
-      audio_only (false) {}
+      audio_only (false),
+      has_played (false) {}
 
 Mrl::~Mrl () {
     if (media_info)
@@ -753,6 +754,10 @@ void Mrl::message (MessageType msg, void *content) {
         }
         break;
 
+    case MsgMediaStarted:
+        has_played = true;
+        break;
+
     case MsgMediaFinished:
         if (state == state_deferred &&
                 !isPlayable () && firstChild ()) {//if backend added child links
@@ -789,6 +794,7 @@ void *Mrl::role (RoleType msg, void *content) {
 }
 
 void Mrl::activate () {
+    has_played = false;
     if (!resolved && isPlayable ()) {
         setState (state_deferred);
         media_info = new MediaInfo (this, ActorAgent::AudioVideo);
