@@ -231,6 +231,7 @@ enum MessageType
     // the above messages are ordered like SMIL timing events
     MsgMediaPrefetch,
     MsgMediaReady,
+    MsgMediaStarted,
     MsgMediaUpdated,
     MsgListUpdated,
     MsgEventPostponed,
@@ -635,6 +636,7 @@ public:
     bool resolved;
     bool bookmarkable;
     bool audio_only;
+    bool has_played;
 };
 
 /**
