@@ -41,7 +41,7 @@ using namespace KMPlayer;
 
 Process::Process (ProcessNotify * ctrl, const char * n, bool xv)
  : m_notify (ctrl), m_viewer (0L), m_name (n),
-      m_state (NotRunning), m_old_state (NotRunning),// m_process (0L),
+      m_state (NotRunning), m_old_state (NotRunning),
       start_timer (0), no_video (false), has_xv (xv), is_local_file (false)
       {}
 
@@ -385,17 +385,26 @@ MPlayer::MPlayer (ProcessNotify *ctrl, const char *nm)
    path ("mplayer"),
    audio_driver (NULL),
    video_driver (NULL),
-   wait_for_pause (false),
+   m_transition_state (NotRunning),
    auto_sub_titles (false) {}
 
 MPlayer::~MPlayer () {
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
+    if (playing ()) {
+        if (node ||
+                (m_transition_state == Paused ||
+                 (Paused == m_state && m_transition_state != Playing))) {
+            m_transition_state = Playing;
+            if (!removeQueued ("pause"))
+                sendCommand (String ("pause"));
+            return true;
+        }
+        return false;
+    }
     debugLog () << "MPlayer::play" << endl;
-    if (playing ())
-        return pause ();
-    wait_for_pause = false;
+    m_transition_state = NotRunning;
     setState (Buffering);
     Process::play (node);
     if (m_url.isEmpty ()) {
@@ -520,14 +529,13 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::pause () {
-    if (state () != Paused)
-        wait_for_pause = !wait_for_pause;
-    for (StringList::iterator i = commands.begin (); i != commands.end (); ++i)
-        if (i->startsWith ("pause")) {
-            commands.erase (i);
-            return true;
-        }
-    return sendCommand (String ("pause"));
+    if (Paused != m_transition_state) {
+        m_transition_state = Paused;
+        if (!removeQueued ("pause"))
+            sendCommand (String ("pause"));
+        return true;
+    }
+    return false;
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::ready (unsigned int viewer) {
@@ -565,16 +573,8 @@ KMPLAYER_NO_EXPORT bool MPlayer::quit () {
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::seek (int pos, bool absolute) {
-    if (/*request_seek != NO_SEEK &&*/ commands.size () > 1) {
-        StringList::iterator i = commands.begin ();
-        StringList::iterator end ( commands.end () );
-        for (++i; i != end; ++i)
-            if ((*i).startsWith (String ("seek"))) {
-                i = commands.erase (i);
-                request_seek = NO_SEEK;
-                break;
-            }
-    }
+    if (removeQueued ("seek"))
+        request_seek = NO_SEEK;
     if (request_seek != NO_SEEK)
         return false;
     if (!absolute && m_length > 0) {
@@ -655,6 +655,8 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
             if (c) {
                 setPosition ((int)(10 * strtof (c + 2, 0L)));
                 request_seek = NO_SEEK;
+                if (Playing == m_transition_state)
+                    m_transition_state = NotRunning;
                 setState (Playing);
             } else {
                 c = strstr (tok, "Cache fill:");
@@ -665,8 +667,11 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
                     //debugLog() << "c " << (c + 11) << endl;
                     m_notify->setLoading ((int)strtof (c + 11, 0L));
                 }
-                if (m_state == Paused)
+                if (m_state == Paused) {
+                    if (Playing == m_transition_state)
+                        m_transition_state = NotRunning;
                     setState (Playing);
+                }
             }
             tok = outbuf + i + 1;
         } else if (outbuf[i] == '\n') {
@@ -706,8 +711,8 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
                         m_length = (int)(10 * strtof (tok + 10, 0L));
                         m_notify->setLength (m_length);
                     } else if (!strncmp (tok + 3, "PAUSED", 6)) {
-                        if (wait_for_pause) {
-                            wait_for_pause = false;
+                        if (Paused == m_transition_state) {
+                            m_transition_state = NotRunning;
                             setState (Paused);
                         } // else ignore, a second 'pause' cmd should follow
                     } else if (!strncmp (tok + 3, "VIDEO_WIDTH", 11)) {
@@ -769,6 +774,16 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
     outbuf[inbuf] = 0;
 }
 
+bool MPlayer::removeQueued (const char *cmd)
+{
+    for (StringList::iterator i = commands.begin (); i != commands.end (); ++i)
+        if (i->startsWith (cmd)) {
+            commands.erase (i);
+            return true;
+        }
+    return false;
+}
+
 KMPLAYER_NO_EXPORT bool MPlayer::sendCommand (const String & cmd) {
     if (playing ()) {
         commands.push_back (cmd + Char ('\n'));
