@@ -94,7 +94,6 @@ protected:
     NodePtrW m_mrl;
     State m_state;
     State m_old_state;
-    //KProcess * m_process;
     String m_url;
     ProcessNotify * m_notify;
     unsigned int m_viewer;
@@ -195,6 +194,7 @@ public:
 
 protected:
     bool sendCommand (const String &);
+    bool removeQueued (const char *cmd);
     typedef std::list <String> StringList;
     StringList commands;
     String command_chunk;
@@ -203,8 +203,8 @@ protected:
     const char *path;
     const char *audio_driver;
     const char *video_driver;
+    State m_transition_state;
     bool request_quit;
-    bool wait_for_pause;
     bool auto_sub_titles;
     bool padding;
     char outbuf[512];
