@@ -38,7 +38,7 @@ static gchar *plugin;
 static gchar *object_url;
 static gchar *mimetype;
 static GtkWidget *xembed;
-static Window socket_id;
+static Window gtk_socket_id;
 static long parent_id;
 static gboolean plugin_embedded;
 
@@ -229,7 +229,7 @@ static void npFinish (void *inst, uint32_t stream, void *ndata) {
     destroyStream (inst, stream);
 }
 
-static gint streamCompare (gconstpointer a, gconstpointer b) {
+static gint pointerCompare (gconstpointer a, gconstpointer b) {
     return (long)a - (long)b;
 }
 
@@ -288,7 +288,7 @@ static void quit ()
 static bool play ()
 {
     nppInstanceTopUrl (object_url);
-    npp_streams = g_tree_new (streamCompare);
+    npp_streams = g_tree_new (pointerCompare);
 
     fprintf (stderr, "play %s\n", object_url);
 
@@ -326,7 +326,7 @@ static gboolean initPlayer (void * p)
         }
         g_tree_insert (npp_plugins, (char *) mimetype, nplib);
     }
-    nppInstanceWindowParent (npdata, (void *) socket_id);
+    nppInstanceWindowParent (npdata, (void *) gtk_socket_id);
     nppInstanceWindowDimension (npdata,
             0, 0, parent_id ? 640 : 565, parent_id ? 480 : 424);
 
@@ -471,7 +471,7 @@ static gboolean configureEvent(GtkWidget *w, GdkEventConfigure *e, gpointer d)
 
 static void windowCreatedEvent (GtkWidget *w, gpointer d) {
     (void)d;
-    socket_id = gtk_socket_get_id (GTK_SOCKET (xembed));
+    gtk_socket_id = gtk_socket_get_id (GTK_SOCKET (xembed));
     if (parent_id) {
         if (!GTK_PLUG (w)->socket_window)
             gtk_plug_construct (GTK_PLUG (w), parent_id);
