@@ -150,7 +150,7 @@ class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     cairo_surface_t * cairo_surface_widget;
     Matrix matrix;
     // stack vars need for transitions
-    SMIL::MediaType *cur_media;
+    TransitionModule *cur_transition;
     cairo_pattern_t * cur_pat;
     cairo_matrix_t cur_mat;
     SMIL::RegionBase::BackgroundRepeat bg_repeat;
@@ -163,7 +163,8 @@ public:
 
     void traverseRegion (Node *reg, Surface *s);
     void updateExternal (SMIL::MediaType *av, SurfacePtr s);
-    void paint(SMIL::MediaType *, Surface *, const IPoint &p, const IRect &);
+    void paint (TransitionModule *trans, MediaOpacity mopacity, Surface *s,
+                const IPoint &p, const IRect &);
     void video (Mrl *mt, Surface *s);
 
     CairoPaintVisitor (ViewArea *v, cairo_surface_t * cs, Matrix m,
@@ -175,7 +176,6 @@ public:
     void visit (SMIL::Layout *);
     void visit (SMIL::RegionBase *);
     void visit (SMIL::Transition *);
-    void visit (SMIL::ImageMediaType *);
     void visit (SMIL::TextMediaType *);
     void visit (SMIL::Brush *);
     void visit (SMIL::SmilText *);
@@ -486,8 +486,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
     }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
-    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_media->trans_gain;
-    if (cur_media->trans_out_active)
+    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_transition->trans_gain;
+    if (cur_transition->trans_out_active)
         perc = 1.0 - perc;
     if (SMIL::Transition::Fade == trans->type) {
         CAIRO_SET_PATTERN_COND(cr, cur_pat, cur_mat)
@@ -639,7 +639,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
 KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
     if (m->media_info &&
             m->media_info->media &&
-            ActorAgent::AudioVideo == m->media_info->type) {
+            (ActorAgent::Audio == m->media_info->type ||
+             ActorAgent::AudioVideo == m->media_info->type)) {
         AudioVideoActor *avm = static_cast<AudioVideoActor *> (m->media_info->media);
         if (s && avm && strcmp (m->nodeName (), "audio")) {
             s->xscale = s->yscale = 1; // either scale width/heigt or use bounds
@@ -652,25 +653,54 @@ KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
+    if (!ref->media_info)
+        return;
     Surface *s = ref->surface ();
-    if (s && ref->external_tree)
+    if (s && ref->external_tree) {
         updateExternal (ref, s);
-    else
+        return;
+    }
+    if (ref->media_info->media &&
+            ref->media_info->media->type () == ActorAgent::Image) {
+        if (!s)
+            return;
+
+        IRect scr = matrix.toScreen (s->bounds);
+        IRect clip_rect = clip.intersect (scr);
+        if (clip_rect.isEmpty ())
+            return;
+
+        ImageActor *im = static_cast <ImageActor *> (ref->media_info->media);
+        ImageData *id = im ? im->cached_img.ptr () : NULL;
+        if (id && id->flags == ImageData::ImageScalable)
+            penalty += im->render (cairo_surface, scr.size);
+        if (!id || im->isEmpty () || ref->size.isEmpty ()) {
+            s->remove();
+            return;
+        }
+        if (!s->surface || s->dirty)
+            id->copyImage (s, SSize (scr.width (), scr.height ()), cairo_surface, &penalty, ref->pan_zoom);
+        paint (&ref->transition, ref->media_opacity, s, scr.point, clip_rect);
+        s->dirty = false;
+
+    } else {
         video (ref, s);
+    }
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
+KDE_NO_EXPORT void CairoPaintVisitor::paint (TransitionModule *trans,
+        MediaOpacity mopacity, Surface *s,
         const IPoint &point, const IRect &rect) {
     cairo_save (cr);
     opacity = 1.0;
     cairo_matrix_init_translate (&cur_mat, -point.x, -point.y);
     cur_pat = cairo_pattern_create_for_surface (s->surface);
-    if (mt->active_trans) {
+    if (trans->active_trans) {
         IRect clip_save = clip;
         clip = rect;
         penalty += rect.width() * rect.height() / (20 * 480);
-        cur_media = mt;
-        mt->active_trans->accept (this);
+        cur_transition = trans;
+        trans->active_trans->accept (this);
         clip = clip_save;
     } else {
         cairo_pattern_set_extend (cur_pat, CAIRO_EXTEND_NONE);
@@ -679,7 +709,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
         cairo_set_source (cr, cur_pat);
         cairo_rectangle (cr, rect.x(), rect.y(), rect.width(), rect.height());
     }
-    opacity *= mt->opacity / 100.0;
+    opacity *= mopacity.opacity / 100.0;
     bool over = opacity < 0.99 ||
                 CAIRO_CONTENT_COLOR != cairo_surface_get_content (s->surface);
     cairo_operator_t op;
@@ -709,7 +739,8 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
              mrl->id < RP::id_node_last);
         if (*rp_or_smil ||
                 (mrl->media_info &&
-                 ActorAgent::AudioVideo == mrl->media_info->type))
+                 (ActorAgent::Audio == mrl->media_info->type ||
+                  ActorAgent::AudioVideo == mrl->media_info->type)))
             return mrl;
     }
     for (Node *c = n->firstChild (); c; c = c->nextSibling ())
@@ -751,39 +782,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
         penalty += visitor.penalty;
         s->dirty = false;
     }
-    paint (av, s.ptr (), scr.point, clip_rect);
-}
-
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
-    if (!img->media_info)
-        return;
-    Surface *s = img->surface ();
-    if (!s)
-        return;
-    if (img->external_tree) {
-        updateExternal (img, s);
-        return;
-    }
-    if (!img->media_info->media)
-        return;
-
-    IRect scr = matrix.toScreen (s->bounds);
-    IRect clip_rect = clip.intersect (scr);
-    if (clip_rect.isEmpty ())
-        return;
-
-    ImageActor *im = static_cast <ImageActor *> (img->media_info->media);
-    ImageData *id = im ? im->cached_img.ptr () : NULL;
-    if (id && id->flags == ImageData::ImageScalable)
-        penalty += im->render (cairo_surface, scr.size);
-    if (!id || im->isEmpty () || img->size.isEmpty ()) {
-        s->remove();
-        return;
-    }
-    if (!s->surface || s->dirty)
-        id->copyImage (s, SSize (scr.width (), scr.height ()), cairo_surface, &penalty, img->pan_zoom);
-    paint (img, s, scr.point, clip_rect);
-    s->dirty = false;
+    paint (&av->transition, av->media_opacity, s.ptr (), scr.point, clip_rect);
 }
 
 static void calculateTextDimensions (PangoFontDescription *desc,
@@ -811,6 +810,30 @@ static void calculateTextDimensions (PangoFontDescription *desc,
     cairo_surface_destroy (img_surf);
 }
 
+static cairo_t *createContext (cairo_surface_t *similar, Surface *s, int w, int h) {
+    unsigned int bg_alpha = s->background_color & 0xff000000;
+    bool clear = s->surface;
+    if (!s->surface)
+        s->surface = cairo_surface_create_similar (similar,
+                bg_alpha < 0xff000000
+                ? CAIRO_CONTENT_COLOR_ALPHA
+                : CAIRO_CONTENT_COLOR,
+                w, h);
+    cairo_t *cr = cairo_create (s->surface);
+    if (clear)
+        clearSurface (cr, IRect (0, 0, w, h));
+    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+
+    if (bg_alpha) {
+        if (bg_alpha < 0xff000000)
+            CAIRO_SET_SOURCE_ARGB (cr, s->background_color);
+        else
+            CAIRO_SET_SOURCE_RGB (cr, s->background_color);
+        cairo_paint (cr);
+    }
+    return cr;
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
     if (!txt->media_info || !txt->media_info->media)
         return;
@@ -841,25 +864,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
             calculateTextDimensions (desc, text,
                     w, 2 * ft_size, &pxw, &pxh, false);
         }
-        unsigned int bg_alpha = s->background_color & 0xff000000;
-        if (!s->surface)
-            s->surface = cairo_surface_create_similar (cairo_surface,
-                    bg_alpha < 0xff000000
-                    ? CAIRO_CONTENT_COLOR_ALPHA
-                    : CAIRO_CONTENT_COLOR,
-                    pxw, pxh);
-        cairo_t *cr_txt = cairo_create (s->surface);
-        if (clear)
-            clearSurface (cr_txt, IRect (0, 0, pxw, pxh));
-        cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
-
-        if (bg_alpha) {
-            if (bg_alpha < 0xff000000)
-                CAIRO_SET_SOURCE_ARGB (cr_txt, s->background_color);
-            else
-                CAIRO_SET_SOURCE_RGB (cr_txt, s->background_color);
-            cairo_paint (cr_txt);
-        }
+        cairo_t *cr_txt = createContext (cairo_surface, s, pxw, pxh);
 
         CAIRO_SET_SOURCE_RGB (cr_txt, txt->font_color);
         PangoLayout *layout = pango_cairo_create_layout (cr_txt);
@@ -884,7 +889,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
     }
     IRect clip_rect = clip.intersect (scr);
     if (!clip_rect.isEmpty ())
-        paint (txt, s, scr.point, clip_rect);
+        paint (&txt->transition, txt->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
 
@@ -896,15 +901,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
         if (clip_rect.isEmpty ())
             return;
         cairo_save (cr);
-        if (brush->active_trans) {
-            cur_media = brush;
+        if (brush->transition.active_trans) {
+            cur_transition = &brush->transition;
             cur_pat = NULL;
-            brush->active_trans->accept (this);
+            brush->transition.active_trans->accept (this);
         } else {
             cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
                     clip_rect.width (), clip_rect.height ());
         }
-        opacity *= brush->opacity / 100.0;
+        opacity *= brush->media_opacity.opacity / 100.0;
         if (opacity < 0.99) {
             cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
             cairo_set_source_rgba (cr,
@@ -1078,6 +1083,7 @@ void SmilTextVisitor::visit (TextNode *text) {
 void SmilTextVisitor::visit (SMIL::TextFlow *flow) {
     if (flow->firstChild ()) {
         bool new_block = SMIL::id_node_p == flow->id ||
+            SMIL::id_node_br == flow->id ||
             SMIL::id_node_div == flow->id;
         float fs = info.props.font_size;
         if (fs < 0)
@@ -1128,10 +1134,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
 
         info.push ();
         if (info.first) {
-            s->surface = cairo_surface_create_similar (cairo_surface,
-                    CAIRO_CONTENT_COLOR_ALPHA, (int) w, info.voffset);
-            cairo_t *cr_txt = cairo_create (s->surface);
-            cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
+            cairo_t *cr_txt = createContext (cairo_surface, s, (int) w, info.voffset);
 
             CAIRO_SET_SOURCE_RGB (cr_txt, 0);
             SmilTextBlock *b = info.first;
@@ -1165,22 +1168,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
         }
     }
     IRect clip_rect = clip.intersect (scr);
-    if (!clip_rect.isEmpty ()) {
-        cairo_save (cr);
-        cairo_matrix_init_translate (&cur_mat, -scr.x (), -scr.y ());
-        cairo_pattern_t *pat = cairo_pattern_create_for_surface (s->surface);
-        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
-        cairo_pattern_set_matrix (pat, &cur_mat);
-        cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
-        cairo_set_source (cr, pat);
-        cairo_rectangle(cr, clip_rect.x (), clip_rect.y (),
-                clip_rect.width (), clip_rect.height ());
-        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-        cairo_fill (cr);
-        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-        cairo_pattern_destroy (pat);
-        cairo_restore (cr);
-    }
+    if (!clip_rect.isEmpty ())
+        paint (&txt->transition, txt->media_opacity, s, scr.point, clip_rect);
     s->dirty = false;
 }
 
